@@ -1626,8 +1626,14 @@ void OBSPropertiesView::AddProperty(obs_property_t *property,
 			QHBoxLayout *boxLayout = new QHBoxLayout(newWidget);
 			boxLayout->setContentsMargins(0, 0, 0, 0);
 			boxLayout->setAlignment(Qt::AlignLeft);
+#ifdef __APPLE__
+			/* TODO: This fixes the issue of tooltip not aligning
+			 * correcty on macOS, the root cause needs further
+			 * investigation. */
+			boxLayout->setSpacing(10);
+#else
 			boxLayout->setSpacing(0);
-
+#endif
 			QCheckBox *check = qobject_cast<QCheckBox *>(widget);
 			check->setText(desc);
 			check->setToolTip(
