@@ -8,7 +8,7 @@ class QLabel;
 class QLineEdit;
 class QListWidget;
 class QListWidgetItem;
-class VisibilityCheckBox;
+class QCheckBox;
 
 class VisibilityItemWidget : public QWidget {
 	Q_OBJECT
@@ -16,7 +16,7 @@ class VisibilityItemWidget : public QWidget {
 private:
 	OBSSource source;
 	QLabel *label = nullptr;
-	VisibilityCheckBox *vis = nullptr;
+	QCheckBox *vis = nullptr;
 	QString oldName;
 
 	OBSSignal enabledSignal;
