@@ -1417,6 +1417,10 @@ void VolumeMeter::paintEvent(QPaintEvent *event)
 
 	QPainter painter(this);
 
+	// Paint window background color (as widget is opaque)
+	QColor background = palette().color(QPalette::ColorRole::Window);
+	painter.fillRect(event->region().boundingRect(), background);
+
 	if (vertical)
 		height -= METER_PADDING * 2;
 
@@ -1426,11 +1430,6 @@ void VolumeMeter::paintEvent(QPaintEvent *event)
 		if (needLayoutChange())
 			doLayout();
 
-		// Paint window background color (as widget is opaque)
-		QColor background =
-			palette().color(QPalette::ColorRole::Window);
-		painter.fillRect(widgetRect, background);
-
 		if (vertical) {
 			paintVTicks(painter,
 				    displayNrAudioChannels *
