@@ -235,6 +235,7 @@ private:
 	QStringList oldExtraDockNames;
 
 	OBSDataAutoRelease safeModeModuleData;
+	std::vector<OBSDataAutoRelease> safeModeTransitions;
 
 	bool loaded = false;
 	long disableSaving = 1;
@@ -359,6 +360,7 @@ private:
 	std::string patronJson;
 
 	std::atomic<obs_scene_t *> currentScene = nullptr;
+	std::optional<std::pair<uint32_t, uint32_t>> lastOutputResolution;
 
 	void UpdateMultiviewProjectorMenu();
 
@@ -1040,6 +1042,7 @@ public:
 	void SetDisplayAffinity(QWindow *window);
 
 	QColor GetSelectionColor() const;
+	inline bool Closing() { return closing; }
 
 protected:
 	virtual void closeEvent(QCloseEvent *event) override;
