@@ -19,6 +19,10 @@
 #include "serializer.h"
 #include "darray.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct array_output_data {
 	DARRAY(uint8_t) bytes;
 };
@@ -26,3 +30,7 @@ struct array_output_data {
 EXPORT void array_output_serializer_init(struct serializer *s,
 					 struct array_output_data *data);
 EXPORT void array_output_serializer_free(struct array_output_data *data);
+
+#ifdef __cplusplus
+}
+#endif
