@@ -158,6 +158,7 @@ static const char *capture_type_to_string(enum portal_capture_type capture_type)
 static void on_pipewire_remote_opened_cb(GObject *source, GAsyncResult *res,
 					 void *user_data)
 {
+	struct obs_pipwire_connect_stream_info connect_info;
 	struct screencast_portal_capture *capture;
 	g_autoptr(GUnixFDList) fd_list = NULL;
 	g_autoptr(GVariant) result = NULL;
@@ -187,19 +188,25 @@ static void on_pipewire_remote_opened_cb(GObject *source, GAsyncResult *res,
 		return;
 	}
 
-	capture->obs_pw = obs_pipewire_create(pipewire_fd);
+	capture->obs_pw = obs_pipewire_connect_fd(pipewire_fd, NULL, NULL);
 
 	if (!capture->obs_pw)
 		return;
 
+	connect_info = (struct obs_pipwire_connect_stream_info){
+		.stream_name = "OBS Studio",
+		.stream_properties = pw_properties_new(
+			PW_KEY_MEDIA_TYPE, "Video", PW_KEY_MEDIA_CATEGORY,
+			"Capture", PW_KEY_MEDIA_ROLE, "Screen", NULL),
+		.screencast =
+			{
+				.cursor_visible = capture->cursor_visible,
+			},
+	};
+
 	capture->obs_pw_stream = obs_pipewire_connect_stream(
 		capture->obs_pw, capture->source, capture->pipewire_node,
-		"OBS Studio",
-		pw_properties_new(PW_KEY_MEDIA_TYPE, "Video",
-				  PW_KEY_MEDIA_CATEGORY, "Capture",
-				  PW_KEY_MEDIA_ROLE, "Screen", NULL));
-	obs_pipewire_stream_set_cursor_visible(capture->obs_pw_stream,
-					       capture->cursor_visible);
+		&connect_info);
 }
 
 static void open_pipewire_remote(struct screencast_portal_capture *capture)
