@@ -1,6 +1,6 @@
 #include <obs-module.h>
 #include <util/platform.h>
-#include <util/circlebuf.h>
+#include <util/deque.h>
 #include <util/dstr.h>
 #include <util/threading.h>
 #include <inttypes.h>
@@ -57,7 +57,7 @@ struct rtmp_stream {
 	obs_output_t *output;
 
 	pthread_mutex_t packets_mutex;
-	struct circlebuf packets;
+	struct deque packets;
 	bool sent_headers;
 
 	bool got_first_video;
@@ -96,14 +96,14 @@ struct rtmp_stream {
 	int dropped_frames;
 
 #ifdef TEST_FRAMEDROPS
-	struct circlebuf droptest_info;
+	struct deque droptest_info;
 	uint64_t droptest_last_key_check;
 	size_t droptest_max;
 	size_t droptest_size;
 #endif
 
 	pthread_mutex_t dbr_mutex;
-	struct circlebuf dbr_frames;
+	struct deque dbr_frames;
 	size_t dbr_data_size;
 	uint64_t dbr_inc_timeout;
 	long audio_bitrate;
