@@ -3,57 +3,61 @@
 #include <dxgi.h>
 
 #include "graphics-hook.h"
-#include "../funchook.h"
 #include "d3d9-patches.hpp"
 
-typedef HRESULT (STDMETHODCALLTYPE *present_t)(IDirect3DDevice9*,
-		CONST RECT*, CONST RECT*, HWND, CONST RGNDATA*);
-typedef HRESULT (STDMETHODCALLTYPE *present_ex_t)(IDirect3DDevice9*,
-		CONST RECT*, CONST RECT*, HWND, CONST RGNDATA*, DWORD);
-typedef HRESULT (STDMETHODCALLTYPE *present_swap_t)(IDirect3DSwapChain9*,
-		CONST RECT*, CONST RECT*, HWND, CONST RGNDATA*, DWORD);
-typedef HRESULT (STDMETHODCALLTYPE *reset_t)(IDirect3DDevice9*,
-		D3DPRESENT_PARAMETERS*);
-typedef HRESULT (STDMETHODCALLTYPE *reset_ex_t)(IDirect3DDevice9*,
-		D3DPRESENT_PARAMETERS*, D3DDISPLAYMODEEX*);
+#include <detours.h>
 
-typedef HRESULT (WINAPI *createfactory1_t)(REFIID, void **);
+typedef HRESULT(STDMETHODCALLTYPE *present_t)(IDirect3DDevice9 *, CONST RECT *,
+					      CONST RECT *, HWND,
+					      CONST RGNDATA *);
+typedef HRESULT(STDMETHODCALLTYPE *present_ex_t)(IDirect3DDevice9Ex *,
+						 CONST RECT *, CONST RECT *,
+						 HWND, CONST RGNDATA *, DWORD);
+typedef HRESULT(STDMETHODCALLTYPE *present_swap_t)(IDirect3DSwapChain9 *,
+						   CONST RECT *, CONST RECT *,
+						   HWND, CONST RGNDATA *,
+						   DWORD);
+typedef HRESULT(STDMETHODCALLTYPE *reset_t)(IDirect3DDevice9 *,
+					    D3DPRESENT_PARAMETERS *);
+typedef HRESULT(STDMETHODCALLTYPE *reset_ex_t)(IDirect3DDevice9 *,
+					       D3DPRESENT_PARAMETERS *,
+					       D3DDISPLAYMODEEX *);
 
-static struct func_hook present;
-static struct func_hook present_ex;
-static struct func_hook present_swap;
-static struct func_hook reset;
-static struct func_hook reset_ex;
+typedef HRESULT(WINAPI *createfactory1_t)(REFIID, void **);
+
+present_t RealPresent = NULL;
+present_ex_t RealPresentEx = NULL;
+present_swap_t RealPresentSwap = NULL;
+reset_t RealReset = NULL;
+reset_ex_t RealResetEx = NULL;
 
 struct d3d9_data {
-	HMODULE                d3d9;
-	IDirect3DDevice9       *device; /* do not release */
-	uint32_t               cx;
-	uint32_t               cy;
-	D3DFORMAT              d3d9_format;
-	DXGI_FORMAT            dxgi_format;
-	bool                   using_shtex;
-	bool                   using_scale;
+	HMODULE d3d9;
+	IDirect3DDevice9 *device; /* do not release */
+	uint32_t cx;
+	uint32_t cy;
+	D3DFORMAT d3d9_format;
+	DXGI_FORMAT dxgi_format;
+	bool using_shtex;
 
 	/* shared texture */
-	IDirect3DSurface9      *d3d9_copytex;
-	ID3D11Device           *d3d11_device;
-	ID3D11DeviceContext    *d3d11_context;
-	ID3D11Resource         *d3d11_tex;
-	struct shtex_data      *shtex_info;
-	HANDLE                 handle;
-	int                    patch;
+	IDirect3DSurface9 *d3d9_copytex;
+	ID3D11Device *d3d11_device;
+	ID3D11DeviceContext *d3d11_context;
+	ID3D11Resource *d3d11_tex;
+	struct shtex_data *shtex_info;
+	HANDLE handle;
+	int patch;
 
 	/* shared memory */
-	IDirect3DSurface9      *copy_surfaces[NUM_BUFFERS];
-	IDirect3DSurface9      *render_targets[NUM_BUFFERS];
-	IDirect3DQuery9        *queries[NUM_BUFFERS];
-	struct shmem_data      *shmem_info;
-	bool                   texture_mapped[NUM_BUFFERS];
-	volatile bool          issued_queries[NUM_BUFFERS];
-	uint32_t               pitch;
-	int                    cur_tex;
-	int                    copy_wait;
+	IDirect3DSurface9 *copy_surfaces[NUM_BUFFERS];
+	IDirect3DQuery9 *queries[NUM_BUFFERS];
+	struct shmem_data *shmem_info;
+	bool texture_mapped[NUM_BUFFERS];
+	volatile bool issued_queries[NUM_BUFFERS];
+	uint32_t pitch;
+	int cur_tex;
+	int copy_wait;
 };
 
 static struct d3d9_data data = {};
@@ -78,8 +82,6 @@ static void d3d9_free()
 					data.copy_surfaces[i]->UnlockRect();
 				data.copy_surfaces[i]->Release();
 			}
-			if (data.render_targets[i])
-				data.render_targets[i]->Release();
 			if (data.queries[i])
 				data.queries[i]->Release();
 		}
@@ -92,21 +94,22 @@ static void d3d9_free()
 
 static DXGI_FORMAT d3d9_to_dxgi_format(D3DFORMAT format)
 {
-	switch ((unsigned long)format) {
-	case D3DFMT_A2B10G10R10: return DXGI_FORMAT_R10G10B10A2_UNORM;
-	case D3DFMT_A8R8G8B8:    return DXGI_FORMAT_B8G8R8A8_UNORM;
-	case D3DFMT_X8R8G8B8:    return DXGI_FORMAT_B8G8R8X8_UNORM;
+	switch (format) {
+	case D3DFMT_A2B10G10R10:
+		return DXGI_FORMAT_R10G10B10A2_UNORM;
+	case D3DFMT_A8R8G8B8:
+		return DXGI_FORMAT_B8G8R8A8_UNORM;
+	case D3DFMT_X8R8G8B8:
+		return DXGI_FORMAT_B8G8R8X8_UNORM;
 	}
 
 	return DXGI_FORMAT_UNKNOWN;
 }
 
-const static D3D_FEATURE_LEVEL feature_levels[] =
-{
+const static D3D_FEATURE_LEVEL feature_levels[] = {
 	D3D_FEATURE_LEVEL_11_0,
 	D3D_FEATURE_LEVEL_10_1,
 	D3D_FEATURE_LEVEL_10_0,
-	D3D_FEATURE_LEVEL_9_3,
 };
 
 static inline bool shex_init_d3d11()
@@ -132,21 +135,21 @@ static inline bool shex_init_d3d11()
 		return false;
 	}
 
-	create_factory = (createfactory1_t)GetProcAddress(dxgi,
-			"CreateDXGIFactory1");
+	create_factory =
+		(createfactory1_t)GetProcAddress(dxgi, "CreateDXGIFactory1");
 	if (!create_factory) {
 		hlog("d3d9_init: Failed to get CreateDXGIFactory1 address");
 		return false;
 	}
 
-	create_device = (PFN_D3D11_CREATE_DEVICE)GetProcAddress(d3d11,
-			"D3D11CreateDevice");
+	create_device = (PFN_D3D11_CREATE_DEVICE)GetProcAddress(
+		d3d11, "D3D11CreateDevice");
 	if (!create_device) {
 		hlog("d3d9_init: Failed to get D3D11CreateDevice address");
 		return false;
 	}
 
-	hr = create_factory(__uuidof(IDXGIFactory1), (void**)&factory);
+	hr = create_factory(__uuidof(IDXGIFactory1), (void **)&factory);
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_init: Failed to create factory object", hr);
 		return false;
@@ -160,11 +163,11 @@ static inline bool shex_init_d3d11()
 		return false;
 	}
 
-	hr = create_device(adapter, D3D_DRIVER_TYPE_UNKNOWN, nullptr,
-			0, feature_levels,
-			sizeof(feature_levels) / sizeof(D3D_FEATURE_LEVEL),
-			D3D11_SDK_VERSION, &data.d3d11_device, &level_used,
-			&data.d3d11_context);
+	hr = create_device(adapter, D3D_DRIVER_TYPE_UNKNOWN, nullptr, 0,
+			   feature_levels,
+			   sizeof(feature_levels) / sizeof(D3D_FEATURE_LEVEL),
+			   D3D11_SDK_VERSION, &data.d3d11_device, &level_used,
+			   &data.d3d11_context);
 	adapter->Release();
 
 	if (FAILED(hr)) {
@@ -181,30 +184,29 @@ static inline bool d3d9_shtex_init_shtex()
 	HRESULT hr;
 
 	D3D11_TEXTURE2D_DESC desc = {};
-	desc.Width                = data.cx;
-	desc.Height               = data.cy;
-	desc.Format               = data.dxgi_format;
-	desc.MipLevels            = 1;
-	desc.ArraySize            = 1;
-	desc.SampleDesc.Count     = 1;
-	desc.Usage                = D3D11_USAGE_DEFAULT;
-	desc.MiscFlags            = D3D11_RESOURCE_MISC_SHARED;
-	desc.BindFlags            = D3D11_BIND_RENDER_TARGET |
-	                            D3D11_BIND_SHADER_RESOURCE;
+	desc.Width = data.cx;
+	desc.Height = data.cy;
+	desc.Format = data.dxgi_format;
+	desc.MipLevels = 1;
+	desc.ArraySize = 1;
+	desc.SampleDesc.Count = 1;
+	desc.Usage = D3D11_USAGE_DEFAULT;
+	desc.MiscFlags = D3D11_RESOURCE_MISC_SHARED;
+	desc.BindFlags = D3D11_BIND_RENDER_TARGET | D3D11_BIND_SHADER_RESOURCE;
 
-	hr = data.d3d11_device->CreateTexture2D(&desc, nullptr,
-			(ID3D11Texture2D**)&data.d3d11_tex);
+	hr = data.d3d11_device->CreateTexture2D(
+		&desc, nullptr, (ID3D11Texture2D **)&data.d3d11_tex);
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shtex_init_shtex: Failed to create D3D11 texture",
-				hr);
+			hr);
 		return false;
 	}
 
 	hr = data.d3d11_tex->QueryInterface(__uuidof(IDXGIResource),
-			(void**)&res);
+					    (void **)&res);
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shtex_init_shtex: Failed to query IDXGIResource",
-				hr);
+			hr);
 		return false;
 	}
 
@@ -213,7 +215,7 @@ static inline bool d3d9_shtex_init_shtex()
 
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shtex_init_shtex: Failed to get shared handle",
-				hr);
+			hr);
 		return false;
 	}
 
@@ -233,10 +235,10 @@ static inline bool d3d9_shtex_init_copytex()
 	HRESULT hr;
 
 	if (offsets.d3d9_clsoff && offsets.is_d3d9ex_clsoff) {
-		uint8_t *device_ptr = (uint8_t*)(data.device);
+		uint8_t *device_ptr = (uint8_t *)(data.device);
 		uint8_t *d3d9_ptr =
-			*(uint8_t**)(device_ptr + offsets.d3d9_clsoff);
-		p_is_d3d9 = (BOOL*)(d3d9_ptr + offsets.is_d3d9ex_clsoff);
+			*(uint8_t **)(device_ptr + offsets.d3d9_clsoff);
+		p_is_d3d9 = (BOOL *)(d3d9_ptr + offsets.is_d3d9ex_clsoff);
 	} else {
 		patch_addr = get_d3d9_patch_addr(data.d3d9, data.patch);
 	}
@@ -248,14 +250,14 @@ static inline bool d3d9_shtex_init_copytex()
 	} else if (patch_addr) {
 		patch_size = patch[data.patch].size;
 		VirtualProtect(patch_addr, patch_size, PAGE_EXECUTE_READWRITE,
-				&protect_val);
+			       &protect_val);
 		memcpy(saved_data, patch_addr, patch_size);
 		memcpy(patch_addr, patch[data.patch].data, patch_size);
 	}
 
 	hr = data.device->CreateTexture(data.cx, data.cy, 1,
-			D3DUSAGE_RENDERTARGET, data.d3d9_format,
-			D3DPOOL_DEFAULT, &tex, &data.handle);
+					D3DUSAGE_RENDERTARGET, data.d3d9_format,
+					D3DPOOL_DEFAULT, &tex, &data.handle);
 
 	if (p_is_d3d9) {
 		*p_is_d3d9 = was_d3d9ex;
@@ -263,12 +265,12 @@ static inline bool d3d9_shtex_init_copytex()
 	} else if (patch_addr && patch_size) {
 		memcpy(patch_addr, saved_data, patch_size);
 		VirtualProtect(patch_addr, patch_size, protect_val,
-				&protect_val);
+			       &protect_val);
 	}
 
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shtex_init_copytex: Failed to create shared texture",
-				hr);
+			hr);
 		return false;
 	}
 
@@ -276,14 +278,15 @@ static inline bool d3d9_shtex_init_copytex()
 	tex->Release();
 
 	if (FAILED(hr)) {
-		hlog_hr("d3d9_shtex_init_copytex: Failed to get surface level", hr);
+		hlog_hr("d3d9_shtex_init_copytex: Failed to get surface level",
+			hr);
 		return false;
 	}
 
 	return true;
 }
 
-static bool d3d9_shtex_init(uint32_t cx, uint32_t cy, HWND window)
+static bool d3d9_shtex_init(HWND window)
 {
 	data.using_shtex = true;
 
@@ -296,8 +299,8 @@ static bool d3d9_shtex_init(uint32_t cx, uint32_t cy, HWND window)
 	if (!d3d9_shtex_init_copytex()) {
 		return false;
 	}
-	if (!capture_init_shtex(&data.shtex_info, window, cx, cy,
-				data.cx, data.cy, data.dxgi_format, false,
+	if (!capture_init_shtex(&data.shtex_info, window, data.cx, data.cy,
+				data.dxgi_format, false,
 				(uintptr_t)data.handle)) {
 		return false;
 	}
@@ -310,40 +313,32 @@ static bool d3d9_shmem_init_buffers(size_t buffer)
 {
 	HRESULT hr;
 
-	hr = data.device->CreateOffscreenPlainSurface(data.cx, data.cy,
-			data.d3d9_format, D3DPOOL_SYSTEMMEM,
-			&data.copy_surfaces[buffer], nullptr);
+	hr = data.device->CreateOffscreenPlainSurface(
+		data.cx, data.cy, data.d3d9_format, D3DPOOL_SYSTEMMEM,
+		&data.copy_surfaces[buffer], nullptr);
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shmem_init_buffers: Failed to create surface",
-				hr);
+			hr);
 		return false;
 	}
 
 	if (buffer == 0) {
 		D3DLOCKED_RECT rect;
 		hr = data.copy_surfaces[buffer]->LockRect(&rect, nullptr,
-				D3DLOCK_READONLY);
+							  D3DLOCK_READONLY);
 		if (FAILED(hr)) {
 			hlog_hr("d3d9_shmem_init_buffers: Failed to lock "
-			        "buffer", hr);
+				"buffer",
+				hr);
 			return false;
 		}
 
 		data.pitch = rect.Pitch;
 		data.copy_surfaces[buffer]->UnlockRect();
-	}	
-
-	hr = data.device->CreateRenderTarget(data.cx, data.cy,
-			data.d3d9_format, D3DMULTISAMPLE_NONE, 0, false,
-			&data.render_targets[buffer], nullptr);
-	if (FAILED(hr)) {
-		hlog_hr("d3d9_shmem_init_buffers: Failed to create render "
-		        "target", hr);
-		return false;
 	}
 
 	hr = data.device->CreateQuery(D3DQUERYTYPE_EVENT,
-			&data.queries[buffer]);
+				      &data.queries[buffer]);
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_shmem_init_buffers: Failed to create query", hr);
 		return false;
@@ -352,7 +347,7 @@ static bool d3d9_shmem_init_buffers(size_t buffer)
 	return true;
 }
 
-static bool d3d9_shmem_init(uint32_t cx, uint32_t cy, HWND window)
+static bool d3d9_shmem_init(HWND window)
 {
 	data.using_shtex = false;
 
@@ -361,9 +356,8 @@ static bool d3d9_shmem_init(uint32_t cx, uint32_t cy, HWND window)
 			return false;
 		}
 	}
-	if (!capture_init_shmem(&data.shmem_info, window, cx, cy,
-				data.cx, data.cy, data.pitch, data.dxgi_format,
-				false)) {
+	if (!capture_init_shmem(&data.shmem_info, window, data.cx, data.cy,
+				data.pitch, data.dxgi_format, false)) {
 		return false;
 	}
 
@@ -387,15 +381,15 @@ static bool d3d9_get_swap_desc(D3DPRESENT_PARAMETERS &pp)
 
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_get_swap_desc: Failed to get "
-		        "presentation parameters", hr);
+			"presentation parameters",
+			hr);
 		return false;
 	}
 
 	return true;
 }
 
-static bool d3d9_init_format_backbuffer(uint32_t &cx, uint32_t &cy,
-		HWND &window)
+static bool d3d9_init_format_backbuffer(HWND &window)
 {
 	IDirect3DSurface9 *back_buffer = nullptr;
 	D3DPRESENT_PARAMETERS pp;
@@ -416,29 +410,22 @@ static bool d3d9_init_format_backbuffer(uint32_t &cx, uint32_t &cy,
 
 	if (FAILED(hr)) {
 		hlog_hr("d3d9_init_format_backbuffer: Failed to get "
-		        "backbuffer descriptor", hr);
+			"backbuffer descriptor",
+			hr);
 		return false;
 	}
 
 	data.d3d9_format = desc.Format;
 	data.dxgi_format = d3d9_to_dxgi_format(desc.Format);
-	data.using_scale = global_hook_info->use_scale;
 	window = pp.hDeviceWindow;
-	cx = desc.Width;
-	cy = desc.Height;
 
-	if (data.using_scale) {
-		data.cx = global_hook_info->cx;
-		data.cy = global_hook_info->cy;
-	} else {
-		data.cx = desc.Width;
-		data.cy = desc.Height;
-	}
+	data.cx = desc.Width;
+	data.cy = desc.Height;
 
 	return true;
 }
 
-static bool d3d9_init_format_swapchain(uint32_t &cx, uint32_t &cy, HWND &window)
+static bool d3d9_init_format_swapchain(HWND &window)
 {
 	D3DPRESENT_PARAMETERS pp;
 
@@ -448,18 +435,10 @@ static bool d3d9_init_format_swapchain(uint32_t &cx, uint32_t &cy, HWND &window)
 
 	data.dxgi_format = d3d9_to_dxgi_format(pp.BackBufferFormat);
 	data.d3d9_format = pp.BackBufferFormat;
-	data.using_scale = global_hook_info->use_scale;
 	window = pp.hDeviceWindow;
-	cx = pp.BackBufferWidth;
-	cy = pp.BackBufferHeight;
 
-	if (data.using_scale) {
-		data.cx = global_hook_info->cx;
-		data.cy = global_hook_info->cy;
-	} else {
-		data.cx = pp.BackBufferWidth;
-		data.cy = pp.BackBufferHeight;
-	}
+	data.cx = pp.BackBufferWidth;
+	data.cy = pp.BackBufferHeight;
 
 	return true;
 }
@@ -471,8 +450,6 @@ static void d3d9_init(IDirect3DDevice9 *device)
 		global_hook_info->offsets.d3d9.d3d9_clsoff &&
 		global_hook_info->offsets.d3d9.is_d3d9ex_clsoff;
 	bool success;
-	uint32_t cx = 0;
-	uint32_t cy = 0;
 	HWND window = nullptr;
 	HRESULT hr;
 
@@ -480,7 +457,7 @@ static void d3d9_init(IDirect3DDevice9 *device)
 	data.device = device;
 
 	hr = device->QueryInterface(__uuidof(IDirect3DDevice9Ex),
-			(void**)&d3d9ex);
+				    (void **)&d3d9ex);
 	if (SUCCEEDED(hr)) {
 		d3d9ex->Release();
 		data.patch = -1;
@@ -490,17 +467,17 @@ static void d3d9_init(IDirect3DDevice9 *device)
 		data.patch = -1;
 	}
 
-	if (!d3d9_init_format_backbuffer(cx, cy, window)) {
-		if (!d3d9_init_format_swapchain(cx, cy, window)) {
+	if (!d3d9_init_format_backbuffer(window)) {
+		if (!d3d9_init_format_swapchain(window)) {
 			return;
 		}
 	}
 
 	if (global_hook_info->force_shmem ||
 	    (!d3d9ex && data.patch == -1 && !has_d3d9ex_bool_offset)) {
-		success = d3d9_shmem_init(cx, cy, window);
+		success = d3d9_shmem_init(window);
 	} else {
-		success = d3d9_shtex_init(cx, cy, window);
+		success = d3d9_shtex_init(window);
 	}
 
 	if (!success)
@@ -508,7 +485,7 @@ static void d3d9_init(IDirect3DDevice9 *device)
 }
 
 static inline HRESULT get_backbuffer(IDirect3DDevice9 *device,
-		IDirect3DSurface9 **surface)
+				     IDirect3DSurface9 **surface)
 {
 	static bool use_backbuffer = false;
 	static bool checked_exceptions = false;
@@ -521,7 +498,7 @@ static inline HRESULT get_backbuffer(IDirect3DDevice9 *device,
 
 	if (use_backbuffer) {
 		return device->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO,
-				surface);
+					     surface);
 	} else {
 		return device->GetRenderTarget(0, surface);
 	}
@@ -529,90 +506,72 @@ static inline HRESULT get_backbuffer(IDirect3DDevice9 *device,
 
 static inline void d3d9_shtex_capture(IDirect3DSurface9 *backbuffer)
 {
-	D3DTEXTUREFILTERTYPE filter;
 	HRESULT hr;
 
-	filter = data.using_scale ? D3DTEXF_LINEAR : D3DTEXF_NONE;
-
 	hr = data.device->StretchRect(backbuffer, nullptr, data.d3d9_copytex,
-			nullptr, filter);
+				      nullptr, D3DTEXF_NONE);
 	if (FAILED(hr))
 		hlog_hr("d3d9_shtex_capture: StretchRect failed", hr);
 }
 
-static inline void d3d9_shmem_capture_queue_copy()
+static void d3d9_shmem_capture_copy(int i)
 {
-	for (int i = 0; i < NUM_BUFFERS; i++) {
-		IDirect3DSurface9 *target = data.copy_surfaces[i];
-		D3DLOCKED_RECT rect;
-		HRESULT hr;
+	IDirect3DSurface9 *target = data.copy_surfaces[i];
+	D3DLOCKED_RECT rect;
+	HRESULT hr;
 
-		if (!data.issued_queries[i]) {
-			continue;
-		}
-		if (data.queries[i]->GetData(0, 0, 0) != S_OK) {
-			continue;
-		}
+	if (!data.issued_queries[i]) {
+		return;
+	}
+	if (data.queries[i]->GetData(0, 0, 0) != S_OK) {
+		return;
+	}
 
-		data.issued_queries[i] = false;
+	data.issued_queries[i] = false;
 
-		hr = target->LockRect(&rect, nullptr, D3DLOCK_READONLY);
-		if (SUCCEEDED(hr)) {
-			data.texture_mapped[i] = true;
-			shmem_copy_data(i, rect.pBits);
-		}
-		break;
+	hr = target->LockRect(&rect, nullptr, D3DLOCK_READONLY);
+	if (SUCCEEDED(hr)) {
+		data.texture_mapped[i] = true;
+		shmem_copy_data(i, rect.pBits);
 	}
 }
 
 static inline void d3d9_shmem_capture(IDirect3DSurface9 *backbuffer)
 {
-	D3DTEXTUREFILTERTYPE filter;
-	IDirect3DSurface9 *copy;
 	int next_tex;
 	HRESULT hr;
 
-	d3d9_shmem_capture_queue_copy();
-
-	next_tex = (data.cur_tex == NUM_BUFFERS - 1) ?  0 : data.cur_tex + 1;
-	filter = data.using_scale ? D3DTEXF_LINEAR : D3DTEXF_NONE;
-	copy = data.render_targets[data.cur_tex];
-
-	hr = data.device->StretchRect(backbuffer, nullptr, copy, nullptr,
-			filter);
-
-	if (FAILED(hr)) {
-		hlog_hr("d3d9_shmem_capture: StretchRect failed", hr);
-		return;
-	}
+	next_tex = (data.cur_tex + 1) % NUM_BUFFERS;
+	d3d9_shmem_capture_copy(next_tex);
 
 	if (data.copy_wait < NUM_BUFFERS - 1) {
 		data.copy_wait++;
 	} else {
-		IDirect3DSurface9 *src = data.render_targets[next_tex];
-		IDirect3DSurface9 *dst = data.copy_surfaces[next_tex];
+		IDirect3DSurface9 *src = backbuffer;
+		IDirect3DSurface9 *dst = data.copy_surfaces[data.cur_tex];
 
-		if (shmem_texture_data_lock(next_tex)) {
+		if (shmem_texture_data_lock(data.cur_tex)) {
 			dst->UnlockRect();
-			data.texture_mapped[next_tex] = false;
-			shmem_texture_data_unlock(next_tex);
+			data.texture_mapped[data.cur_tex] = false;
+			shmem_texture_data_unlock(data.cur_tex);
 		}
 
 		hr = data.device->GetRenderTargetData(src, dst);
 		if (FAILED(hr)) {
 			hlog_hr("d3d9_shmem_capture: GetRenderTargetData "
-			        "failed", hr);
+				"failed",
+				hr);
 		}
 
-		data.queries[next_tex]->Issue(D3DISSUE_END);
-		data.issued_queries[next_tex] = true;
+		data.queries[data.cur_tex]->Issue(D3DISSUE_END);
+		data.issued_queries[data.cur_tex] = true;
 	}
 
 	data.cur_tex = next_tex;
 }
 
 static void d3d9_capture(IDirect3DDevice9 *device,
-		IDirect3DSurface9 *backbuffer)
+			 IDirect3DSurface9 *backbuffer)
 {
 	if (capture_should_stop()) {
 		d3d9_free();
@@ -637,7 +596,7 @@ static void d3d9_capture(IDirect3DDevice9 *device,
 static int present_recurse = 0;
 
 static inline void present_begin(IDirect3DDevice9 *device,
-		IDirect3DSurface9 *&backbuffer)
+				 IDirect3DSurface9 *&backbuffer)
 {
 	HRESULT hr;
 
@@ -645,7 +604,8 @@ static inline void present_begin(IDirect3DDevice9 *device,
 		hr = get_backbuffer(device, &backbuffer);
 		if (FAILED(hr)) {
 			hlog_hr("d3d9_shmem_capture: Failed to get "
-				"backbuffer", hr);
+				"backbuffer",
+				hr);
 		}
 
 		if (!global_hook_info->capture_overlay) {
@@ -657,7 +617,7 @@ static inline void present_begin(IDirect3DDevice9 *device,
 }
 
 static inline void present_end(IDirect3DDevice9 *device,
-		IDirect3DSurface9 *backbuffer)
+			       IDirect3DSurface9 *backbuffer)
 {
 	present_recurse--;
 
@@ -676,62 +636,56 @@ static bool hooked_reset = false;
 static void setup_reset_hooks(IDirect3DDevice9 *device);
 
 static HRESULT STDMETHODCALLTYPE hook_present(IDirect3DDevice9 *device,
-		CONST RECT *src_rect, CONST RECT *dst_rect,
-		HWND override_window, CONST RGNDATA *dirty_region)
+					      CONST RECT *src_rect,
+					      CONST RECT *dst_rect,
+					      HWND override_window,
+					      CONST RGNDATA *dirty_region)
 {
 	IDirect3DSurface9 *backbuffer = nullptr;
-	HRESULT hr;
 
 	if (!hooked_reset)
 		setup_reset_hooks(device);
 
 	present_begin(device, backbuffer);
 
-	unhook(&present);
-	present_t call = (present_t)present.call_addr;
-	hr = call(device, src_rect, dst_rect, override_window, dirty_region);
-	rehook(&present);
+	const HRESULT hr = RealPresent(device, src_rect, dst_rect,
+				       override_window, dirty_region);
 
 	present_end(device, backbuffer);
 
 	return hr;
 }
 
-static HRESULT STDMETHODCALLTYPE hook_present_ex(IDirect3DDevice9 *device,
-		CONST RECT *src_rect, CONST RECT *dst_rect,
-		HWND override_window, CONST RGNDATA *dirty_region, DWORD flags)
+static HRESULT STDMETHODCALLTYPE hook_present_ex(
+	IDirect3DDevice9Ex *device, CONST RECT *src_rect, CONST RECT *dst_rect,
+	HWND override_window, CONST RGNDATA *dirty_region, DWORD flags)
 {
 	IDirect3DSurface9 *backbuffer = nullptr;
-	HRESULT hr;
 
 	if (!hooked_reset)
 		setup_reset_hooks(device);
 
 	present_begin(device, backbuffer);
 
-	unhook(&present_ex);
-	present_ex_t call = (present_ex_t)present_ex.call_addr;
-	hr = call(device, src_rect, dst_rect, override_window, dirty_region,
-			flags);
-	rehook(&present_ex);
+	const HRESULT hr = RealPresentEx(device, src_rect, dst_rect,
+					 override_window, dirty_region, flags);
 
 	present_end(device, backbuffer);
 
 	return hr;
 }
 
-static HRESULT STDMETHODCALLTYPE hook_present_swap(IDirect3DSwapChain9 *swap,
-		CONST RECT *src_rect, CONST RECT *dst_rect,
-		HWND override_window, CONST RGNDATA *dirty_region, DWORD flags)
+static HRESULT STDMETHODCALLTYPE hook_present_swap(
+	IDirect3DSwapChain9 *swap, CONST RECT *src_rect, CONST RECT *dst_rect,
+	HWND override_window, CONST RGNDATA *dirty_region, DWORD flags)
 {
 	IDirect3DSurface9 *backbuffer = nullptr;
 	IDirect3DDevice9 *device = nullptr;
-	HRESULT hr;
 
 	if (!present_recurse) {
-		hr = swap->GetDevice(&device);
-		if (SUCCEEDED(hr)) {
-			device->Release();
+		IDirect3DDevice9 *temp;
+		if (SUCCEEDED(swap->GetDevice(&temp))) {
+			device = temp;
 		}
 	}
 
@@ -742,79 +696,73 @@ static HRESULT STDMETHODCALLTYPE hook_present_swap(IDirect3DSwapChain9 *swap,
 		present_begin(device, backbuffer);
 	}
 
-	unhook(&present_swap);
-	present_swap_t call = (present_swap_t)present_swap.call_addr;
-	hr = call(swap, src_rect, dst_rect, override_window, dirty_region,
-			flags);
-	rehook(&present_swap);
+	const HRESULT hr = RealPresentSwap(
+		swap, src_rect, dst_rect, override_window, dirty_region, flags);
 
-	if (device)
+	if (device) {
 		present_end(device, backbuffer);
+		device->Release();
+	}
 
 	return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE hook_reset(IDirect3DDevice9 *device,
-		D3DPRESENT_PARAMETERS *params)
+					    D3DPRESENT_PARAMETERS *params)
 {
-	HRESULT hr;
-
 	if (capture_active())
 		d3d9_free();
 
-	unhook(&reset);
-	reset_t call = (reset_t)reset.call_addr;
-	hr = call(device, params);
-	rehook(&reset);
-
-	return hr;
+	return RealReset(device, params);
 }
 
 static HRESULT STDMETHODCALLTYPE hook_reset_ex(IDirect3DDevice9 *device,
-		D3DPRESENT_PARAMETERS *params, D3DDISPLAYMODEEX *dmex)
+					       D3DPRESENT_PARAMETERS *params,
+					       D3DDISPLAYMODEEX *dmex)
 {
-	HRESULT hr;
-
 	if (capture_active())
 		d3d9_free();
 
-	unhook(&reset_ex);
-	reset_ex_t call = (reset_ex_t)reset_ex.call_addr;
-	hr = call(device, params, dmex);
-	rehook(&reset_ex);
-
-	return hr;
+	return RealResetEx(device, params, dmex);
 }
 
 static void setup_reset_hooks(IDirect3DDevice9 *device)
 {
 	IDirect3DDevice9Ex *d3d9ex = nullptr;
-	uintptr_t *vtable = *(uintptr_t**)device;
+	uintptr_t *vtable = *(uintptr_t **)device;
 	HRESULT hr;
 
-	hook_init(&reset, (void*)vtable[16], (void*)hook_reset,
-			"IDirect3DDevice9::Reset");
-	rehook(&reset);
+	DetourTransactionBegin();
 
-	hr = device->QueryInterface(__uuidof(IDirect3DDevice9Ex),
-			(void**)&d3d9ex);
+	RealReset = (reset_t)vtable[16];
+	DetourAttach((PVOID *)&RealReset, hook_reset);
+
+	hr = device->QueryInterface(IID_PPV_ARGS(&d3d9ex));
 	if (SUCCEEDED(hr)) {
-		hook_init(&reset_ex, (void*)vtable[132], (void*)hook_reset_ex,
-				"IDirect3DDevice9Ex::ResetEx");
-		rehook(&reset_ex);
+		RealResetEx = (reset_ex_t)vtable[132];
+		DetourAttach((PVOID *)&RealResetEx, hook_reset_ex);
 
 		d3d9ex->Release();
 	}
 
-	hooked_reset = true;
+	const LONG error = DetourTransactionCommit();
+	const bool success = error == NO_ERROR;
+	if (success) {
+		hlog("Hooked IDirect3DDevice9::Reset");
+		if (RealResetEx)
+			hlog("Hooked IDirect3DDevice9Ex::ResetEx");
+		hooked_reset = true;
+	} else {
+		RealReset = nullptr;
+		RealResetEx = nullptr;
+	}
 }
 
-typedef HRESULT (WINAPI *d3d9create_ex_t)(UINT, IDirect3D9Ex**);
+typedef HRESULT(WINAPI *d3d9create_ex_t)(UINT, IDirect3D9Ex **);
 
-static bool manually_get_d3d9_addrs(HMODULE d3d9_module,
-		void **present_addr,
-		void **present_ex_addr,
-		void **present_swap_addr)
+static bool manually_get_d3d9_addrs(HMODULE d3d9_module, void **present_addr,
+				    void **present_ex_addr,
+				    void **present_swap_addr)
 {
 	d3d9create_ex_t create_ex;
 	D3DPRESENT_PARAMETERS pp;
@@ -826,7 +774,7 @@ static bool manually_get_d3d9_addrs(HMODULE d3d9_module,
 	hlog("D3D9 values invalid, manually obtaining");
 
 	create_ex = (d3d9create_ex_t)GetProcAddress(d3d9_module,
-			"Direct3DCreate9Ex");
+						    "Direct3DCreate9Ex");
 	if (!create_ex) {
 		hlog("Failed to load Direct3DCreate9Ex");
 		return false;
@@ -837,30 +785,30 @@ static bool manually_get_d3d9_addrs(HMODULE d3d9_module,
 	}
 
 	memset(&pp, 0, sizeof(pp));
-	pp.Windowed                 = 1;
-	pp.SwapEffect               = D3DSWAPEFFECT_FLIP;
-	pp.BackBufferFormat         = D3DFMT_A8R8G8B8;
-	pp.BackBufferCount          = 1;
-	pp.hDeviceWindow            = (HWND)dummy_window;
-	pp.PresentationInterval     = D3DPRESENT_INTERVAL_IMMEDIATE;
+	pp.Windowed = 1;
+	pp.SwapEffect = D3DSWAPEFFECT_FLIP;
+	pp.BackBufferFormat = D3DFMT_A8R8G8B8;
+	pp.BackBufferCount = 1;
+	pp.hDeviceWindow = (HWND)dummy_window;
+	pp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
 
-	hr = d3d9ex->CreateDeviceEx(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
-			dummy_window,
-			D3DCREATE_HARDWARE_VERTEXPROCESSING |
-			D3DCREATE_NOWINDOWCHANGES, &pp, NULL, &device);
+	hr = d3d9ex->CreateDeviceEx(
+		D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, dummy_window,
+		D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_NOWINDOWCHANGES,
+		&pp, NULL, &device);
 	d3d9ex->Release();
 
 	if (SUCCEEDED(hr)) {
-		uintptr_t *vtable = *(uintptr_t**)device;
+		uintptr_t *vtable = *(uintptr_t **)device;
 		IDirect3DSwapChain9 *swap;
 
-		*present_addr = (void*)vtable[17];
-		*present_ex_addr = (void*)vtable[121];
+		*present_addr = (void *)vtable[17];
+		*present_ex_addr = (void *)vtable[121];
 
 		hr = device->GetSwapChain(0, &swap);
 		if (SUCCEEDED(hr)) {
-			vtable = *(uintptr_t**)swap;
-			*present_swap_addr = (void*)vtable[3];
+			vtable = *(uintptr_t **)swap;
+			*present_swap_addr = (void *)vtable[3];
 
 			swap->Release();
 		}
@@ -888,25 +836,25 @@ bool hook_d3d9(void)
 
 	d3d9_size = module_size(d3d9_module);
 
-	if (global_hook_info->offsets.d3d9.present      < d3d9_size &&
-	    global_hook_info->offsets.d3d9.present_ex   < d3d9_size &&
+	if (global_hook_info->offsets.d3d9.present < d3d9_size &&
+	    global_hook_info->offsets.d3d9.present_ex < d3d9_size &&
 	    global_hook_info->offsets.d3d9.present_swap < d3d9_size) {
 
-		present_addr = get_offset_addr(d3d9_module,
-				global_hook_info->offsets.d3d9.present);
-		present_ex_addr = get_offset_addr(d3d9_module,
-				global_hook_info->offsets.d3d9.present_ex);
-		present_swap_addr = get_offset_addr(d3d9_module,
-				global_hook_info->offsets.d3d9.present_swap);
+		present_addr = get_offset_addr(
+			d3d9_module, global_hook_info->offsets.d3d9.present);
+		present_ex_addr = get_offset_addr(
+			d3d9_module, global_hook_info->offsets.d3d9.present_ex);
+		present_swap_addr = get_offset_addr(
+			d3d9_module,
+			global_hook_info->offsets.d3d9.present_swap);
 	} else {
 		if (!dummy_window) {
 			return false;
 		}
 
-		if (!manually_get_d3d9_addrs(d3d9_module,
-					&present_addr,
-					&present_ex_addr,
-					&present_swap_addr)) {
+		if (!manually_get_d3d9_addrs(d3d9_module, &present_addr,
+					     &present_ex_addr,
+					     &present_swap_addr)) {
 			hlog("Failed to get D3D9 values");
 			return true;
 		}
@@ -917,25 +865,37 @@ bool hook_d3d9(void)
 		return true;
 	}
 
+	DetourTransactionBegin();
+
 	if (present_swap_addr) {
-		hook_init(&present_swap, present_swap_addr,
-				(void*)hook_present_swap,
-				"IDirect3DSwapChain9::Present");
-		rehook(&present_swap);
+		RealPresentSwap = (present_swap_t)present_swap_addr;
+		DetourAttach((PVOID *)&RealPresentSwap, hook_present_swap);
 	}
 	if (present_ex_addr) {
-		hook_init(&present_ex, present_ex_addr,
-				(void*)hook_present_ex,
-				"IDirect3DDevice9Ex::PresentEx");
-		rehook(&present_ex);
+		RealPresentEx = (present_ex_t)present_ex_addr;
+		DetourAttach((PVOID *)&RealPresentEx, hook_present_ex);
 	}
 	if (present_addr) {
-		hook_init(&present, present_addr,
-				(void*)hook_present,
-				"IDirect3DDevice9::Present");
-		rehook(&present);
+		RealPresent = (present_t)present_addr;
+		DetourAttach((PVOID *)&RealPresent, hook_present);
 	}
 
-	hlog("Hooked D3D9");
-	return true;
+	const LONG error = DetourTransactionCommit();
+	const bool success = error == NO_ERROR;
+	if (success) {
+		if (RealPresentSwap)
+			hlog("Hooked IDirect3DSwapChain9::Present");
+		if (RealPresentEx)
+			hlog("Hooked IDirect3DDevice9Ex::PresentEx");
+		if (RealPresent)
+			hlog("Hooked IDirect3DDevice9::Present");
+		hlog("Hooked D3D9");
+	} else {
+		RealPresentSwap = nullptr;
+		RealPresentEx = nullptr;
+		RealPresent = nullptr;
+		hlog("Failed to attach Detours hook: %ld", error);
+	}
+
+	return success;
 }
